library travis_dart;

import "package:http/http.dart";
import "package:dartson/dartson.dart";
import "dart:async";

import "models/get/repository.dart";
import "models/get/user.dart";
import "models/get/build.dart";
import "models/get/error.dart";
import "models/get/organization.dart";
import "models/get/job.dart";
import "result.dart";

const TRAVIS_API_URL = "https://api.travis-ci.org";

class Travis {
  final String token;

  Travis(this.token);

  Future<Result<T, Errors>> _request<T>(
      String route, HttpMethod httpMethod, T result,
      [String content]) async {
    var headers = {"Authorization": "token $token", "Travis-API-Version": 3};
    var dson = new Dartson.JSON();
    Response response;
    switch (httpMethod) {
      case HttpMethod.Get:
        response = await get("$TRAVIS_API_URL/$route", headers: headers);
        break;
      case HttpMethod.Put:
        response = await put("$TRAVIS_API_URL/$route",
            headers: headers, body: content);
        break;
      case HttpMethod.Post:
        response = await post("$TRAVIS_API_URL/$route",
            headers: headers, body: content);
        break;
      case HttpMethod.Delete:
        response = await delete("$TRAVIS_API_URL/$route", headers: headers);
        break;
      default:
        throw new Exception("invalid HttpMethod");
    }
    if (response.statusCode < 200 || response.statusCode > 299)
      return Result.ofError(dson.decode(response.body, new Errors()));

    return Result.of(dson.decode(response.body, result));
  }

  Future<Result<Repositories, Errors>> getRepositories([String username]) =>
      _request<Repositories>(
          username == null ? "repos" : "owner/$username/repos",
          HttpMethod.Get,
          new Repositories());

  Future<Result<Repositories, Errors>> getRepositoriesByGitHubId(
          int githubId) =>
      _request<Repositories>(
          "owner/github_id/$githubId", HttpMethod.Get, new Repositories());

  Future<Result<Repository, Errors>> getRepository(int repositoryId) =>
      _request<Repository>(
          "repo/$repositoryId", HttpMethod.Get, new Repository());

  Future<Result<Repository, Errors>> getRepositoryBySlug(String slug) =>
      _request<Repository>("repo/$slug", HttpMethod.Get, new Repository());

  Future<Result<Repository, Errors>> activateRepository(int repositoryId) =>
      _request<Repository>(
          "repo/$repositoryId/activate", HttpMethod.Post, new Repository());

  Future<Result<Repository, Errors>> activateRepositoryBySlug(String slug) =>
      _request<Repository>(
          "repo/$slug/activate", HttpMethod.Post, new Repository());

  Future<Result<Repository, Errors>> deactivateRepository(int repositoryId) =>
      _request<Repository>(
          "repo/$repositoryId/deactivate", HttpMethod.Post, new Repository());

  Future<Result<Repository, Errors>> deactivateRepositoryBySlug(String slug) =>
      _request<Repository>(
          "repo/$slug/deactivate", HttpMethod.Post, new Repository());

  Future<Result<Repository, Errors>> starRepository(int repositoryId) =>
      _request<Repository>(
          "repo/$repositoryId/star", HttpMethod.Post, new Repository());

  Future<Result<Repository, Errors>> starRepositoryBySlug(String slug) =>
      _request<Repository>(
          "repo/$slug/star", HttpMethod.Post, new Repository());

  Future<Result<Repository, Errors>> unstarRepository(int repositoryId) =>
      _request<Repository>(
          "repo/$repositoryId/unstar", HttpMethod.Post, new Repository());

  Future<Result<Repository, Errors>> unstarRepositoryBySlug(String slug) =>
      _request<Repository>(
          "repo/$slug/unstar", HttpMethod.Post, new Repository());

  Future<Result<User, Errors>> getUser([int userId]) => _request<User>(
      userId == null ? "user" : "user/$userId", HttpMethod.Get, new User());

  Future<Result<User, Errors>> syncUser(int userId) =>
      _request<User>("user/$userId/sync", HttpMethod.Post, new User());

  Future<Result<Builds, Errors>> getBuilds() =>
      _request("builds", HttpMethod.Get, new Builds());

  Future<Result<Builds, Errors>> getRepositoryBuilds(int repositoryId) =>
      _request<Builds>(
          "repo/$repositoryId/builds", HttpMethod.Get, new Builds());

  Future<Result<Builds, Errors>> getRepositoryBuildsBySlug(String slug) =>
      _request<Builds>("repo/$slug/builds", HttpMethod.Get, new Builds());

  Future<Result<Build, Errors>> getBuild(int buildId) =>
      _request<Build>("build/$buildId", HttpMethod.Get, new Build());

  Future<Result<Build, Errors>> cancelBuild(int buildId) =>
      _request<Build>("build/$buildId/cancel", HttpMethod.Post, new Build());

  Future<Result<Build, Errors>> restartBuild(int buildId) =>
      _request<Build>("build/$buildId/restart", HttpMethod.Post, new Build());

  Future<Result<Organization, Errors>> getOrganization(int organizationId) =>
      _request<Organization>(
          "org/$organizationId", HttpMethod.Get, new Organization());

  Future<Result<Organizations, Errors>> getOrganizations() =>
      _request<Organizations>("orgs", HttpMethod.Get, new Organizations());

  Future<Result<Job, Errors>> getJob(int jobId) =>
      _request<Job>("job/$jobId", HttpMethod.Get, new Job());

  Future<Result<Job, Errors>> cancelJob(int jobId) =>
      _request<Job>("job/$jobId/cancel", HttpMethod.Post, new Job());

  Future<Result<Job, Errors>> restartJob(int jobId) =>
      _request<Job>("job/$jobId/restart", HttpMethod.Post, new Job());

  Future<Result<Job, Errors>> debugJob(int jobId) =>
      _request<Job>("job/$jobId/debug", HttpMethod.Post, new Job());
}

enum HttpMethod { Get, Put, Post, Delete }
